// This file is automatically generated by rmconfig - DO NOT EDIT!
//
// Provides RPCSTRUCTURECOPY object boilerplate and RTTI.
//
// Profile:  shipping-gpus-openrm
// Haldef:   rpcstructurecopy.def
// Template: templates/gt_eng_odb.h
//

#ifndef _G_RPCSTRUCTURECOPY_ODB_H_
#define _G_RPCSTRUCTURECOPY_ODB_H_

#include "core/core.h"
#include "g_rpcstructurecopy_hal.h"


#define __RPCSTRUCTURECOPY_OBJECT_BASE_DEFINITION \
    const struct NVOC_RTTI *__nvoc_rtti; \
    RPCSTRUCTURECOPY_HAL_IFACES _hal


extern const struct NVOC_CLASS_DEF __iom_class_def_OBJRPCSTRUCTURECOPY;
#define __nvoc_class_def_OBJRPCSTRUCTURECOPY __iom_class_def_OBJRPCSTRUCTURECOPY

#ifndef __nvoc_class_id_OBJRPCSTRUCTURECOPY
#define __nvoc_class_id_OBJRPCSTRUCTURECOPY 0xd4dff8
#endif /* __nvoc_class_id_OBJRPCSTRUCTURECOPY */


#endif  // _G_RPCSTRUCTURECOPY_ODB_H_
