/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __ls10_dev_reductiontstate_ip_h__
#define __ls10_dev_reductiontstate_ip_h__
/* This file is autogenerated.  Do not edit */
#define NV_REDUCTIONTSTATE          0x00002fff:0x00002800 /* RW--D */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0                    0x00002c00      /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_TAGPOOL_ECC_LIMIT_ERR 2:2          /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_TAGPOOL_ECC_LIMIT_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_TAGPOOL_ECC_LIMIT_ERR_NONE 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_TAGPOOL_ECC_LIMIT_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_TAGPOOL_ECC_DBE_ERR 3:3            /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_TAGPOOL_ECC_DBE_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_TAGPOOL_ECC_DBE_ERR_NONE 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_TAGPOOL_ECC_DBE_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_BUF_OVERWRITE_ERR 16:16 /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_BUF_OVERWRITE_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_BUF_OVERWRITE_ERR_NONE 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_BUF_OVERWRITE_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_ECC_LIMIT_ERR 18:18     /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_ECC_LIMIT_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_ECC_LIMIT_ERR_NONE 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_ECC_LIMIT_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_ECC_DBE_ERR 19:19       /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_ECC_DBE_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_ECC_DBE_ERR_NONE 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_ECC_DBE_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_RTO_ERR 20:20           /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_RTO_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_RTO_ERR_NONE 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_STATUS_0_CRUMBSTORE_RTO_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0                    0x00002c04      /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_TAGPOOL_ECC_LIMIT_ERR 2:2          /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_TAGPOOL_ECC_LIMIT_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_TAGPOOL_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_TAGPOOL_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_TAGPOOL_ECC_DBE_ERR 3:3            /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_TAGPOOL_ECC_DBE_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_TAGPOOL_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_TAGPOOL_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR 16:16 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_ECC_LIMIT_ERR 18:18     /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_ECC_LIMIT_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_ECC_DBE_ERR 19:19       /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_ECC_DBE_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_RTO_ERR 20:20           /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_RTO_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_RTO_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_LOG_EN_0_CRUMBSTORE_RTO_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0           0x00002c08      /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_TAGPOOL_ECC_LIMIT_ERR 2:2 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_TAGPOOL_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_TAGPOOL_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_TAGPOOL_ECC_DBE_ERR 3:3   /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_TAGPOOL_ECC_DBE_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_TAGPOOL_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_TAGPOOL_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR 16:16 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_LIMIT_ERR 18:18 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_DBE_ERR 19:19 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_DBE_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_RTO_ERR 20:20  /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_RTO_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_FATAL_REPORT_EN_0_CRUMBSTORE_RTO_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0       0x00002c0c      /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_TAGPOOL_ECC_LIMIT_ERR 2:2 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_TAGPOOL_ECC_LIMIT_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_TAGPOOL_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_TAGPOOL_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_TAGPOOL_ECC_DBE_ERR 3:3 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_TAGPOOL_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_TAGPOOL_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR 16:16 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_LIMIT_ERR 18:18 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_LIMIT_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_DBE_ERR 19:19 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_RTO_ERR 20:20 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_RTO_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_RTO_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_NON_FATAL_REPORT_EN_0_CRUMBSTORE_RTO_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_CORRECTABLE_REPORT_EN_0     0x00002c10      /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0                0x00002c14      /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_TAGPOOL_ECC_LIMIT_ERR 2:2      /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_TAGPOOL_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_TAGPOOL_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_TAGPOOL_ECC_DBE_ERR 3:3        /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_TAGPOOL_ECC_DBE_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_TAGPOOL_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_TAGPOOL_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR 16:16 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_BUF_OVERWRITE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_ECC_LIMIT_ERR 18:18 /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_ECC_LIMIT_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_ECC_LIMIT_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_ECC_DBE_ERR 19:19   /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_ECC_DBE_ERR__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_ECC_DBE_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_ECC_DBE_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_RTO_ERR 20:20       /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_RTO_ERR_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_CONTAIN_EN_0_CRUMBSTORE_RTO_ERR_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0                     0x00002c1c      /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_TAGPOOL_ECC_LIMIT_ERR 2:2           /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_TAGPOOL_ECC_LIMIT_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_TAGPOOL_ECC_LIMIT_ERR_NONE 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_TAGPOOL_ECC_LIMIT_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_TAGPOOL_ECC_DBE_ERR 3:3             /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_TAGPOOL_ECC_DBE_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_TAGPOOL_ECC_DBE_ERR_NONE 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_TAGPOOL_ECC_DBE_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_BUF_OVERWRITE_ERR 16:16  /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_BUF_OVERWRITE_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_BUF_OVERWRITE_ERR_NONE 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_BUF_OVERWRITE_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_ECC_LIMIT_ERR 18:18      /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_ECC_LIMIT_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_ECC_LIMIT_ERR_NONE 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_ECC_LIMIT_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_ECC_DBE_ERR 19:19        /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_ECC_DBE_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_ECC_DBE_ERR_NONE 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_ECC_DBE_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_RTO_ERR  20:20           /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_RTO_ERR__ONWRITE "oneToClear" /*       */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_RTO_ERR_NONE 0x00000000  /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_FIRST_0_CRUMBSTORE_RTO_ERR_CLEAR 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_TIMESTAMP_LOG               0x00002c50      /* R--4R */
#define NV_REDUCTIONTSTATE_ERR_TIMESTAMP_LOG_TIMESTAMP     23:0            /* R-DVF */
#define NV_REDUCTIONTSTATE_ERR_TIMESTAMP_LOG_TIMESTAMP_INIT 0x00000000     /* R-D-V */
#define NV_REDUCTIONTSTATE_ERR_ECC_CTRL                    0x00002d00      /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_ECC_CTRL_TAGPOOL_ECC_ENABLE 0:0             /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_ECC_CTRL_TAGPOOL_ECC_ENABLE_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_ECC_CTRL_TAGPOOL_ECC_ENABLE_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_ECC_CTRL_TAGPOOL_ECC_ENABLE__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_ECC_CTRL_CRUMBSTORE_ECC_ENABLE 1:1          /* RWEVF */
#define NV_REDUCTIONTSTATE_ERR_ECC_CTRL_CRUMBSTORE_ECC_ENABLE_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_ECC_CTRL_CRUMBSTORE_ECC_ENABLE_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_ERR_ECC_CTRL_CRUMBSTORE_ECC_ENABLE__PROD 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_COUNTER   0x00002d20      /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_COUNTER_ERROR_COUNT 23:0  /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_COUNTER_ERROR_COUNT_INIT 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_COUNTER_LIMIT 0x00002d40  /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_COUNTER_LIMIT_ERROR_LIMIT 23:0 /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_COUNTER_LIMIT_ERROR_LIMIT_INIT 0x00ffffff /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_ADDRESS   0x00002d60      /* R--4R */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_ADDRESS_ERROR_ADDRESS 7:0 /* R-DVF */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_ADDRESS_ERROR_ADDRESS_INIT 0x00000000 /* R-D-V */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_ADDRESS_VALID 0x00002d80  /* R--4R */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_ADDRESS_VALID_VALID 0:0   /* R-DVF */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_ADDRESS_VALID_VALID_INVALID 0x00000000 /* R-D-V */
#define NV_REDUCTIONTSTATE_ERR_TAGPOOL_ECC_ERROR_ADDRESS_VALID_VALID_VALID 0x00000001 /* R---V */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_COUNTER 0x00002e00     /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_COUNTER_ERROR_COUNT 23:0 /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_COUNTER_ERROR_COUNT_INIT 0x00000000 /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_COUNTER_LIMIT 0x00002e20 /* RW-4R */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_COUNTER_LIMIT_ERROR_LIMIT 23:0 /* RWDVF */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_COUNTER_LIMIT_ERROR_LIMIT_INIT 0x00ffffff /* RWD-V */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_ADDRESS 0x00002e40     /* R--4R */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_ADDRESS_ERROR_ADDRESS 7:0 /* R-DVF */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_ADDRESS_ERROR_ADDRESS_INIT 0x00000000 /* R-D-V */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_ADDRESS_VALID 0x00002e60 /* R--4R */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_ADDRESS_VALID_VALID 0:0 /* R-DVF */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_ADDRESS_VALID_VALID_INVALID 0x00000000 /* R-D-V */
#define NV_REDUCTIONTSTATE_ERR_CRUMBSTORE_ECC_ERROR_ADDRESS_VALID_VALID_VALID 0x00000001 /* R---V */
#define NV_REDUCTIONTSTATE_STAT_STALL_BUSY_CONTROL         0x00002f00      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_STALL_BUSY_CONTROL_ENABLE_TIMER 0:0        /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_STALL_BUSY_CONTROL_ENABLE_TIMER_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_STALL_BUSY_CONTROL_ENABLE_TIMER_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_STAT_STALL_BUSY_CONTROL_SNAP_ON_DEMAND 1:1      /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_STALL_BUSY_CONTROL_SNAP_ON_DEMAND__ONWRITE "oneToSet" /*       */
#define NV_REDUCTIONTSTATE_STAT_STALL_BUSY_CONTROL_SNAP_ON_DEMAND_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_STALL_BUSY_CONTROL_SNAP_ON_DEMAND_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_0_HIGH              0x00002f04      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_0_HIGH_VALUE        15:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_0_HIGH_VALUE_INIT   0x00000000      /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_0_LOW               0x00002f08      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_0_LOW_VALUE         31:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_0_LOW_VALUE_INIT    0x00000000      /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_0_HIGH          0x00002f0c      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_0_HIGH_VALUE    15:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_0_HIGH_VALUE_INIT 0x00000000    /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_0_LOW           0x00002f10      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_0_LOW_VALUE     31:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_0_LOW_VALUE_INIT 0x00000000     /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_0_HIGH         0x00002f14      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_0_HIGH_VALUE   15:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_0_HIGH_VALUE_INIT 0x00000000   /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_0_LOW          0x00002f18      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_0_LOW_VALUE    31:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_0_LOW_VALUE_INIT 0x00000000    /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_1_HIGH              0x00002f1c      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_1_HIGH_VALUE        15:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_1_HIGH_VALUE_INIT   0x00000000      /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_1_LOW               0x00002f20      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_1_LOW_VALUE         31:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_WINDOW_1_LOW_VALUE_INIT    0x00000000      /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_1_HIGH          0x00002f24      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_1_HIGH_VALUE    15:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_1_HIGH_VALUE_INIT 0x00000000    /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_1_LOW           0x00002f28      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_1_LOW_VALUE     31:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_BUSY_TIMER_1_LOW_VALUE_INIT 0x00000000     /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_1_HIGH         0x00002f2c      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_1_HIGH_VALUE   15:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_1_HIGH_VALUE_INIT 0x00000000   /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_1_LOW          0x00002f30      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_1_LOW_VALUE    31:0            /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_STALL_TIMER_1_LOW_VALUE_INIT 0x00000000    /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_CONTROL          0x00002f34      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_CONTROL_ENABLE_TIMER 0:0         /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_CONTROL_ENABLE_TIMER_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_CONTROL_ENABLE_TIMER_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_CONTROL_SNAP_ON_DEMAND 1:1       /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_CONTROL_SNAP_ON_DEMAND__ONWRITE "oneToSet" /*       */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_CONTROL_SNAP_ON_DEMAND_DISABLE 0x00000000 /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_CONTROL_SNAP_ON_DEMAND_ENABLE 0x00000001 /* RW--V */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_BIN_CTRL_LOW     0x00002f38      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_BIN_CTRL_LOW_LIMIT 23:0          /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_BIN_CTRL_LOW_LIMIT_INIT 0x0028b0ab /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_BIN_CTRL_HIGH    0x00002f3c      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_BIN_CTRL_HIGH_LIMIT 23:0         /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_BIN_CTRL_HIGH_LIMIT_INIT 0x00cb7355 /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL       0x00002f40      /* RW-4R */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_INDEX 9:0             /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_INDEX_MIN 0x00000000  /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_INDEX_MAX 0x000002ff  /* RW--V */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_INDEX_MCID_STRIDE 0x00000006 /*       */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_INDEX_BIN_LOW_31_0 0x00000000 /*       */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_INDEX_BIN_LOW_47_32 0x00000001 /*       */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_INDEX_BIN_MED_31_0 0x00000002 /*       */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_INDEX_BIN_MED_47_32 0x00000003 /*       */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_INDEX_BIN_HIGH_31_0 0x00000004 /*       */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_INDEX_BIN_HIGH_47_32 0x00000005 /*       */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_AUTOINCR 10:10        /* RWEVF */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_AUTOINCR_OFF 0x00000000 /* RW--V */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_CTRL_AUTOINCR_ON 0x00000001 /* RWE-V */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_DATA       0x00002f44      /* R--4R */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_DATA_COUNT 31:0            /* R-XVF */
#define NV_REDUCTIONTSTATE_STAT_RESIDENCY_COUNT_DATA__APERTURE_INDEX_OFFSET -0x00000004 /*       */
#endif // __ls10_dev_reductiontstate_ip_h__
