// Generated by the protocol buffer compiler.  DO NOT EDIT!

#ifndef G_JOURNAL_PB_H__
#define G_JOURNAL_PB_H__


extern const PRB_ENUM_DESC prb_enums_journal_assert_rectyp;

// 'Assert.RecTyp' enumeration values
#define JOURNAL_ASSERT_ASSERT 120
#define JOURNAL_ASSERT_TIMEOUT 121
#define JOURNAL_ASSERT_DEBUG_BREAKPOINT 124
#define JOURNAL_ASSERT_ASSERT_V3 139
#define JOURNAL_ASSERT_TIMEOUT_V3 140
#define JOURNAL_ASSERT_DEBUG_BREAKPOINT_V3 141

extern const PRB_ENUM_DESC prb_enums_journal_badread_memspace;

// 'BadRead.MemSpace' enumeration values
#define JOURNAL_BADREAD_BAR0 1
#define JOURNAL_BADREAD_FB 2
#define JOURNAL_BADREAD_INSTANCE 3
#define JOURNAL_BADREAD_PCI 4

extern const PRB_ENUM_DESC prb_enums_journal_badread_reason;

// 'BadRead.Reason' enumeration values
#define JOURNAL_BADREAD_GPU_OFF_BUS 1
#define JOURNAL_BADREAD_LOW_POWER 2
#define JOURNAL_BADREAD_PCI_DEVICE_DISABLED 3
#define JOURNAL_BADREAD_GPU_RESET 4
#define JOURNAL_BADREAD_DWORD_SHIFT 5
#define JOURNAL_BADREAD_UNKNOWN 6

extern const PRB_MSG_DESC prb_messages_journal[];

// Message descriptor pointers
#define JOURNAL_COMMON (&prb_messages_journal[0])
#define JOURNAL_ASSERT (&prb_messages_journal[1])
#define JOURNAL_RVAHEADER (&prb_messages_journal[2])
#define JOURNAL_BADREAD (&prb_messages_journal[3])
#define JOURNAL_BUGCHECK (&prb_messages_journal[4])

// Message maximum lengths
// Does not include repeated fields, strings and byte arrays.
#define JOURNAL_COMMON_LEN 39
#define JOURNAL_ASSERT_LEN 125
#define JOURNAL_RVAHEADER_LEN 51
#define JOURNAL_BADREAD_LEN 67
#define JOURNAL_BUGCHECK_LEN 66

extern const PRB_FIELD_DESC prb_fields_journal_common[];

// 'Common' field descriptor pointers
#define JOURNAL_COMMON_GPU_TAG (&prb_fields_journal_common[0])
#define JOURNAL_COMMON_CPU_TAG (&prb_fields_journal_common[1])
#define JOURNAL_COMMON_TIME_STAMP (&prb_fields_journal_common[2])
#define JOURNAL_COMMON_STATE_MASK (&prb_fields_journal_common[3])

// 'Common' field lengths
#define JOURNAL_COMMON_GPU_TAG_LEN 5
#define JOURNAL_COMMON_CPU_TAG_LEN 10
#define JOURNAL_COMMON_TIME_STAMP_LEN 10
#define JOURNAL_COMMON_STATE_MASK_LEN 10

extern const PRB_FIELD_DESC prb_fields_journal_assert[];

// 'Assert' field descriptor pointers
#define JOURNAL_ASSERT_TYPE (&prb_fields_journal_assert[0])
#define JOURNAL_ASSERT_COMMON_OBSOLETE (&prb_fields_journal_assert[1])
#define JOURNAL_ASSERT_LAST_TIME_STAMP (&prb_fields_journal_assert[2])
#define JOURNAL_ASSERT_FIRST_TIME (&prb_fields_journal_assert[3])
#define JOURNAL_ASSERT_LAST_TIME (&prb_fields_journal_assert[4])
#define JOURNAL_ASSERT_BREAKPOINT_ADDR_HINT (&prb_fields_journal_assert[5])
#define JOURNAL_ASSERT_GPU_TAG (&prb_fields_journal_assert[6])
#define JOURNAL_ASSERT_COUNT (&prb_fields_journal_assert[7])
#define JOURNAL_ASSERT_LEVEL (&prb_fields_journal_assert[8])
#define JOURNAL_ASSERT_CALL_STACK (&prb_fields_journal_assert[9])
#define JOURNAL_ASSERT_SOURCE_LINE (&prb_fields_journal_assert[10])

// 'Assert' field lengths
#define JOURNAL_ASSERT_TYPE_LEN 2
#define JOURNAL_ASSERT_COMMON_OBSOLETE_LEN 42
#define JOURNAL_ASSERT_LAST_TIME_STAMP_LEN 10
#define JOURNAL_ASSERT_FIRST_TIME_LEN 10
#define JOURNAL_ASSERT_LAST_TIME_LEN 10
#define JOURNAL_ASSERT_BREAKPOINT_ADDR_HINT_LEN 10
#define JOURNAL_ASSERT_GPU_TAG_LEN 5
#define JOURNAL_ASSERT_COUNT_LEN 5
#define JOURNAL_ASSERT_LEVEL_LEN 5
#define JOURNAL_ASSERT_CALL_STACK_LEN 10
#define JOURNAL_ASSERT_SOURCE_LINE_LEN 5

extern const PRB_FIELD_DESC prb_fields_journal_rvaheader[];

// 'RvaHeader' field descriptor pointers
#define JOURNAL_RVAHEADER_DRIVER_START (&prb_fields_journal_rvaheader[0])
#define JOURNAL_RVAHEADER_OFFSET (&prb_fields_journal_rvaheader[1])
#define JOURNAL_RVAHEADER_POINTER_SIZE (&prb_fields_journal_rvaheader[2])
#define JOURNAL_RVAHEADER_UNIQUE_ID_HIGH (&prb_fields_journal_rvaheader[3])
#define JOURNAL_RVAHEADER_UNIQUE_ID_LOW (&prb_fields_journal_rvaheader[4])
#define JOURNAL_RVAHEADER_AGE (&prb_fields_journal_rvaheader[5])

// 'RvaHeader' field lengths
#define JOURNAL_RVAHEADER_DRIVER_START_LEN 10
#define JOURNAL_RVAHEADER_OFFSET_LEN 5
#define JOURNAL_RVAHEADER_POINTER_SIZE_LEN 5
#define JOURNAL_RVAHEADER_UNIQUE_ID_HIGH_LEN 10
#define JOURNAL_RVAHEADER_UNIQUE_ID_LOW_LEN 10
#define JOURNAL_RVAHEADER_AGE_LEN 5

extern const PRB_FIELD_DESC prb_fields_journal_badread[];

// 'BadRead' field descriptor pointers
#define JOURNAL_BADREAD_MEMORY_SPACE (&prb_fields_journal_badread[0])
#define JOURNAL_BADREAD_OFFSET (&prb_fields_journal_badread[1])
#define JOURNAL_BADREAD_MASK (&prb_fields_journal_badread[2])
#define JOURNAL_BADREAD_VALUE (&prb_fields_journal_badread[3])
#define JOURNAL_BADREAD_REASON (&prb_fields_journal_badread[4])
#define JOURNAL_BADREAD_COMMON_OBSOLETE (&prb_fields_journal_badread[5])

// 'BadRead' field lengths
#define JOURNAL_BADREAD_MEMORY_SPACE_LEN 2
#define JOURNAL_BADREAD_OFFSET_LEN 5
#define JOURNAL_BADREAD_MASK_LEN 5
#define JOURNAL_BADREAD_VALUE_LEN 5
#define JOURNAL_BADREAD_REASON_LEN 2
#define JOURNAL_BADREAD_COMMON_OBSOLETE_LEN 42

extern const PRB_FIELD_DESC prb_fields_journal_bugcheck[];

// 'BugCheck' field descriptor pointers
#define JOURNAL_BUGCHECK_CODE (&prb_fields_journal_bugcheck[0])
#define JOURNAL_BUGCHECK_GPU_TAG (&prb_fields_journal_bugcheck[1])
#define JOURNAL_BUGCHECK_TIME_STAMP (&prb_fields_journal_bugcheck[2])
#define JOURNAL_BUGCHECK_COMMON_OBSOLETE (&prb_fields_journal_bugcheck[3])

// 'BugCheck' field lengths
#define JOURNAL_BUGCHECK_CODE_LEN 5
#define JOURNAL_BUGCHECK_GPU_TAG_LEN 5
#define JOURNAL_BUGCHECK_TIME_STAMP_LEN 10
#define JOURNAL_BUGCHECK_COMMON_OBSOLETE_LEN 42

extern const PRB_SERVICE_DESC prb_services_journal[];

// Service descriptor pointers

#endif // G_JOURNAL_PB_H__
