/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __lr10_dev_nvs_h__
#define __lr10_dev_nvs_h__
/* This file is autogenerated.  Do not edit */
#define NV_PBUS_PRI_TIMEOUT_SAVE_1                     0x00001988 /* RW-4R */
#define NV_PBUS_PRI_TIMEOUT_SAVE_1_DATA                      31:0 /* RWXVF */
#define NV_PBUS_PRI_TIMEOUT_SAVE_1_DATA_WAS_READ              0x0 /* RW--V */

#define NV_PBUS_PRI_TIMEOUT_SAVE_3                     0x00001990 /* R--4R */
#define NV_PBUS_PRI_TIMEOUT_SAVE_3_SUBID                      3:0 /* R-XVF */

#define NV_PBUS_PRI_TIMEOUT_FECS_ERRCODE               0x0000198C /* RW-4R */
#define NV_PBUS_PRI_TIMEOUT_FECS_ERRCODE_DATA                31:0 /* RWXVF */

#define NV_PBUS_PRI_TIMEOUT_SAVE_0                     0x00001984 /* RW-4R */
#define NV_PBUS_PRI_TIMEOUT_SAVE_0_TO                         0:0 /* RWXVF */
#define NV_PBUS_PRI_TIMEOUT_SAVE_0_TO_ERROR                   0x1 /* RW--V */
#define NV_PBUS_PRI_TIMEOUT_SAVE_0_TO_CLEAR                   0x0 /* -W--V */
#define NV_PBUS_PRI_TIMEOUT_SAVE_0_TO_NONE                    0x0 /* RW--V */
#define NV_PBUS_PRI_TIMEOUT_SAVE_0_WRITE                      1:1 /* RWXVF */
#define NV_PBUS_PRI_TIMEOUT_SAVE_0_WRITE_TRUE                 0x1 /* RW--V */
#define NV_PBUS_PRI_TIMEOUT_SAVE_0_WRITE_FALSE                0x0 /* RW--V */
#define NV_PBUS_PRI_TIMEOUT_SAVE_0_ADDR                      25:2 /* RWXVF */
#define NV_PBUS_PRI_TIMEOUT_SAVE_0_FECS_TGT                 31:31 /* RWXVF */
#define NV_PBUS_PRI_TIMEOUT_SAVE_0_FECS_TGT_TRUE              0x1 /* RW--V */
#define NV_PBUS_PRI_TIMEOUT_SAVE_0_FECS_TGT_FALSE             0x0 /* RW--V */

#define NV_PBUS_EXT_CG1                               0x00001C04 /* RW-4R */
#define NV_PBUS_EXT_CG1_SLCG                                   9:1 /*       */
#define NV_PBUS_EXT_CG1_SLCG_ENABLED                    0x00000000 /*       */
#define NV_PBUS_EXT_CG1_SLCG_DISABLED                   0x000001ff /*       */
#define NV_PBUS_EXT_CG1_SLCG__PROD                      0x00000000 /*       */
#define NV_PBUS_EXT_CG1_SLCG_C11                              2:2 /* RWIVF */
#define NV_PBUS_EXT_CG1_SLCG_C11_ENABLED               0x00000000 /* RW--V */
#define NV_PBUS_EXT_CG1_SLCG_C11_DISABLED              0x00000001 /* RWI-V */
#define NV_PBUS_EXT_CG1_SLCG_C11__PROD                 0x00000000 /* RW--V */
#define NV_PBUS_EXT_CG1_SLCG_PRI                              4:4 /* RWIVF */
#define NV_PBUS_EXT_CG1_SLCG_PRI_ENABLED               0x00000000 /* RW--V */
#define NV_PBUS_EXT_CG1_SLCG_PRI_DISABLED              0x00000001 /* RWI-V */
#define NV_PBUS_EXT_CG1_SLCG_PRI__PROD                 0x00000000 /* RW--V */
#define NV_PBUS_EXT_CG1_SLCG_UNROLL                           5:5 /* RWIVF */
#define NV_PBUS_EXT_CG1_SLCG_UNROLL_ENABLED            0x00000000 /* RW--V */
#define NV_PBUS_EXT_CG1_SLCG_UNROLL_DISABLED           0x00000001 /* RWI-V */
#define NV_PBUS_EXT_CG1_SLCG_UNROLL__PROD              0x00000000 /* RW--V */
#define NV_PBUS_EXT_CG1_SLCG_ROLL                             7:7 /* RWIVF */
#define NV_PBUS_EXT_CG1_SLCG_ROLL_ENABLED               0x00000000 /* RW--V */
#define NV_PBUS_EXT_CG1_SLCG_ROLL_DISABLED              0x00000001 /* RWI-V */
#define NV_PBUS_EXT_CG1_SLCG_ROLL__PROD                 0x00000000 /* RW--V */
#define NV_PBUS_EXT_CG1_SLCG_IFR                               8:8 /* RWIVF */
#define NV_PBUS_EXT_CG1_SLCG_IFR_ENABLED                0x00000000 /* RW--V */
#define NV_PBUS_EXT_CG1_SLCG_IFR_DISABLED               0x00000001 /* RWI-V */
#define NV_PBUS_EXT_CG1_SLCG_IFR__PROD                  0x00000000 /* RW--V */
#define NV_PBUS_EXT_CG1_SLCG_PMC                            10:10 /* RWIVF */
#define NV_PBUS_EXT_CG1_SLCG_PMC_ENABLED               0x00000000 /* RW--V */
#define NV_PBUS_EXT_CG1_SLCG_PMC_DISABLED              0x00000001 /* RWI-V */
#define NV_PBUS_EXT_CG1_SLCG_PMC__PROD                 0x00000000 /* RW--V */

#define NV_PBUS_INTR_EN_0                                0x00001140 /* RW-4R */
#define NV_PBUS_INTR_EN_0_PRI_SQUASH                            1:1 /* RWIVF */
#define NV_PBUS_INTR_EN_0_PRI_SQUASH_DISABLED            0x00000000 /* RWI-V */
#define NV_PBUS_INTR_EN_0_PRI_SQUASH_ENABLED             0x00000001 /* RW--V */
#define NV_PBUS_INTR_EN_0_PRI_FECSERR                           2:2 /* RWIVF */
#define NV_PBUS_INTR_EN_0_PRI_FECSERR_DISABLED           0x00000000 /* RWI-V */
#define NV_PBUS_INTR_EN_0_PRI_FECSERR_ENABLED            0x00000001 /* RW--V */
#define NV_PBUS_INTR_EN_0_PRI_TIMEOUT                           3:3 /* RWIVF */
#define NV_PBUS_INTR_EN_0_PRI_TIMEOUT_DISABLED           0x00000000 /* RWI-V */
#define NV_PBUS_INTR_EN_0_PRI_TIMEOUT_ENABLED            0x00000001 /* RW--V */
#define NV_PBUS_INTR_EN_0_SW                                  26:26 /* RWIVF */
#define NV_PBUS_INTR_EN_0_SW_DISABLED                    0x00000000 /* RWI-V */
#define NV_PBUS_INTR_EN_0_SW_ENABLED                     0x00000001 /* RW--V */

#define NV_PBUS_INTR_0                                   0x00001100 /* RW-4R */
#define NV_PBUS_INTR_0_PRI_SQUASH                               1:1 /* RWIVF */
#define NV_PBUS_INTR_0_PRI_SQUASH_NOT_PENDING            0x00000000 /* R-I-V */
#define NV_PBUS_INTR_0_PRI_SQUASH_PENDING                0x00000001 /* R---V */
#define NV_PBUS_INTR_0_PRI_SQUASH_RESET                  0x00000001 /* -W--C */
#define NV_PBUS_INTR_0_PRI_FECSERR                              2:2 /* RWIVF */
#define NV_PBUS_INTR_0_PRI_FECSERR_NOT_PENDING           0x00000000 /* R-I-V */
#define NV_PBUS_INTR_0_PRI_FECSERR_PENDING               0x00000001 /* R---V */
#define NV_PBUS_INTR_0_PRI_FECSERR_RESET                 0x00000001 /* -W--C */
#define NV_PBUS_INTR_0_PRI_TIMEOUT                              3:3 /* RWIVF */
#define NV_PBUS_INTR_0_PRI_TIMEOUT_NOT_PENDING           0x00000000 /* R-I-V */
#define NV_PBUS_INTR_0_PRI_TIMEOUT_PENDING               0x00000001 /* R---V */
#define NV_PBUS_INTR_0_PRI_TIMEOUT_RESET                 0x00000001 /* -W--C */
#define NV_PBUS_INTR_0_SW                                     26:26 /* RWIVF */
#define NV_PBUS_INTR_0_SW_NOT_PENDING                    0x00000000 /* R-I-V */
#define NV_PBUS_INTR_0_SW_PENDING                        0x00000001 /* R---V */
#define NV_PBUS_INTR_0_SW_RESET                          0x00000001 /* -W--C */
#endif // __lr10_dev_nvs_h__
