// Generated by the protocol buffer compiler.  DO NOT EDIT!

#include "nvtypes.h"
#include "prbrt.h"
#include "g_nvdebug_pb.h"

// 'OsType' enum descriptor
static const PRB_ENUM_MAPPING prb_enum_mappings_ostype[] = {
    {
        1,
        PRB_MAYBE_ENUM_NAME("OS_NT")
    },
    {
        2,
        PRB_MAYBE_ENUM_NAME("OS_UNIX")
    },
    {
        3,
        PRB_MAYBE_ENUM_NAME("OS_MAC")
    },
    {
        4,
        PRB_MAYBE_ENUM_NAME("OS_MODS")
    },
    {
        5,
        PRB_MAYBE_ENUM_NAME("OS_QNX")
    },
    {
        6,
        PRB_MAYBE_ENUM_NAME("OS_UNITTEST")
    },
    {
        7,
        PRB_MAYBE_ENUM_NAME("OS_UNK")
    },
    {
        8,
        PRB_MAYBE_ENUM_NAME("OS_LIBOS")
    },
};

const PRB_ENUM_DESC prb_enums_nvdebug_ostype = {
    prb_enum_mappings_ostype,
    8,
   PRB_MAYBE_ENUM_NAME("OsType")
};

// 'SystemInfo' field defaults

// 'SystemInfo' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo[] = {
    {
        1,
        {
            PRB_OPTIONAL,
            PRB_UINT64,
            0 | PRB_IS_DEPRECATED,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("timestamp")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_OPTIONAL,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_SYSTEMINFO_NORTHBRIDGEINFO,
        0,
        PRB_MAYBE_FIELD_NAME("northbridge_info")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        3,
        {
            PRB_OPTIONAL,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_SYSTEMINFO_CPUINFO,
        0,
        PRB_MAYBE_FIELD_NAME("cpu_info")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        4,
        {
            PRB_OPTIONAL,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_SYSTEMINFO_GPUINFO,
        0,
        PRB_MAYBE_FIELD_NAME("gpu_info")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        5,
        {
            PRB_OPTIONAL,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_SYSTEMINFO_OSINFO,
        0,
        PRB_MAYBE_FIELD_NAME("os_info")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        6,
        {
            PRB_OPTIONAL,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_SYSTEMINFO_DRIVERINFO,
        0,
        PRB_MAYBE_FIELD_NAME("driver_info")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        7,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_SYSTEMINFO_CONFIG,
        0,
        PRB_MAYBE_FIELD_NAME("gpu_config")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        8,
        {
            PRB_OPTIONAL,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_SYSTEMINFO_ERRORSTATE,
        0,
        PRB_MAYBE_FIELD_NAME("error_state")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        9,
        {
            PRB_OPTIONAL,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_SYSTEMINFO_SOCINFO,
        0,
        PRB_MAYBE_FIELD_NAME("soc_info")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        10,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0 | PRB_IS_DEPRECATED,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("time_since_boot")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        11,
        {
            PRB_OPTIONAL,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_SYSTEMINFO_TIMEINFO,
        0,
        PRB_MAYBE_FIELD_NAME("time_info")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        12,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("bugcheck_count")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'GpuInfo' field defaults

// 'GpuInfo' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_gpuinfo[] = {
    {
        1,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_ENG_GPU,
        0,
        PRB_MAYBE_FIELD_NAME("eng_gpu")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_ENG_MC,
        0,
        PRB_MAYBE_FIELD_NAME("eng_mc")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        18,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_ENG_NVD,
        0,
        PRB_MAYBE_FIELD_NAME("eng_nvd")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'NvDump' field defaults

// 'NvDump' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_nvdump[] = {
    {
        1,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_SYSTEMINFO,
        0,
        PRB_MAYBE_FIELD_NAME("system_info")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        DCL_DCLMSG,
        0,
        PRB_MAYBE_FIELD_NAME("dcl_msg")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        3,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_GPUINFO,
        0,
        PRB_MAYBE_FIELD_NAME("gpu_info")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        4,
        {
            PRB_REPEATED,
            PRB_UINT64,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("exception_address")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        101,
        {
            PRB_REPEATED,
            PRB_MESSAGE,
            0,
        },
        NVDEBUG_SYSTEMINFO,
        0,
        PRB_MAYBE_FIELD_NAME("system_info_gsprm")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'NorthBridgeInfo' field defaults

// 'NorthBridgeInfo' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_northbridgeinfo[] = {
    {
        1,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("id")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("ssid")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'SocInfo' field defaults

// 'SocInfo' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_socinfo[] = {
    {
        1,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("vendorId")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("chipId")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'CpuInfo' field defaults

// 'CpuInfo' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_cpuinfo[] = {
    {
        1,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("cpu_type")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("num_cpu_cores")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        3,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("num_logical_cpus")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        4,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("cpu_caps")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'GpuInfo' field defaults

// 'GpuInfo' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_gpuinfo[] = {
    {
        1,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("num_gpus")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_REPEATED,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("gpu_id")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        3,
        {
            PRB_REPEATED,
            PRB_STRING,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("gpu_uuid")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        4,
        {
            PRB_REPEATED,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("device_id")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        5,
        {
            PRB_REPEATED,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("pmcBoot0")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        6,
        {
            PRB_REPEATED,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("junction_temp")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        7,
        {
            PRB_REPEATED,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("subdev_id")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'OsInfo' field defaults

// 'OsInfo' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_osinfo[] = {
    {
        1,
        {
            PRB_OPTIONAL,
            PRB_ENUM,
            0,
        },
        0,
        &prb_enums_nvdebug_ostype,
        PRB_MAYBE_FIELD_NAME("family")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("osMajorVersion")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        3,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("osMinorVersion")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        4,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("osBldNum")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        5,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("winProductType")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        6,
        {
            PRB_OPTIONAL,
            PRB_STRING,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("buildVersion")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        7,
        {
            PRB_OPTIONAL,
            PRB_STRING,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("buildDatePlus")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'DriverInfo' field defaults

// 'DriverInfo' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_driverinfo[] = {
    {
        1,
        {
            PRB_OPTIONAL,
            PRB_BOOL,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("is_release")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_OPTIONAL,
            PRB_STRING,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("version")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        3,
        {
            PRB_OPTIONAL,
            PRB_STRING,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("branch")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        4,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("changelist")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        5,
        {
            PRB_OPTIONAL,
            PRB_STRING,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("previous_version")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        6,
        {
            PRB_OPTIONAL,
            PRB_STRING,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("previous_branch")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        7,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("previous_changelist")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        8,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("load_count")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'Config' field defaults

// 'Config' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_config[] = {
    {
        1,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("master_id")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_REPEATED,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("gpu_id")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'ErrorState' field defaults

// 'ErrorState' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_errorstate[] = {
    {
        1,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("bugcheck_code")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_OPTIONAL,
            PRB_BOOL,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("got_rm_lock")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        3,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("dump_buffer_size")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// 'TimeInfo' field defaults

// 'TimeInfo' field descriptors
const PRB_FIELD_DESC prb_fields_nvdebug_systeminfo_timeinfo[] = {
    {
        1,
        {
            PRB_OPTIONAL,
            PRB_UINT64,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("timestamp_freq")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        2,
        {
            PRB_OPTIONAL,
            PRB_UINT64,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("timestamp_dump")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        3,
        {
            PRB_OPTIONAL,
            PRB_UINT64,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("system_time_dump")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        4,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0 | PRB_IS_DEPRECATED,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("time_since_boot_us")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
    {
        5,
        {
            PRB_OPTIONAL,
            PRB_UINT32,
            0,
        },
        0,
        0,
        PRB_MAYBE_FIELD_NAME("time_since_boot_sec")
        PRB_MAYBE_FIELD_DEFAULT(0)
    },
};

// Message descriptors
const PRB_MSG_DESC prb_messages_nvdebug[] = {
    {
        12,
        prb_fields_nvdebug_systeminfo,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.SystemInfo")
    },
    {
        3,
        prb_fields_nvdebug_gpuinfo,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.GpuInfo")
    },
    {
        5,
        prb_fields_nvdebug_nvdump,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.NvDump")
    },
    {
        2,
        prb_fields_nvdebug_systeminfo_northbridgeinfo,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.SystemInfo.NorthBridgeInfo")
    },
    {
        2,
        prb_fields_nvdebug_systeminfo_socinfo,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.SystemInfo.SocInfo")
    },
    {
        4,
        prb_fields_nvdebug_systeminfo_cpuinfo,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.SystemInfo.CpuInfo")
    },
    {
        7,
        prb_fields_nvdebug_systeminfo_gpuinfo,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.SystemInfo.GpuInfo")
    },
    {
        7,
        prb_fields_nvdebug_systeminfo_osinfo,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.SystemInfo.OsInfo")
    },
    {
        8,
        prb_fields_nvdebug_systeminfo_driverinfo,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.SystemInfo.DriverInfo")
    },
    {
        2,
        prb_fields_nvdebug_systeminfo_config,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.SystemInfo.Config")
    },
    {
        3,
        prb_fields_nvdebug_systeminfo_errorstate,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.SystemInfo.ErrorState")
    },
    {
        5,
        prb_fields_nvdebug_systeminfo_timeinfo,
        PRB_MAYBE_MESSAGE_NAME("NvDebug.SystemInfo.TimeInfo")
    },
};

// Service descriptors
const PRB_SERVICE_DESC prb_services_nvdebug[] = {
   { 0 }
};

