/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __ls10_dev_nvlphyctl_ip_h__
#define __ls10_dev_nvlphyctl_ip_h__
/* This file is autogenerated.  Do not edit */
#define NV_NVLPHYCTL                 0x00002DFF:0x00002800 /* RW--D */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS              0x00002840      /* RW-4R */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMREQ        3:0             /* RWEVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMREQ_NOP    0x00000000      /* RWE-V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMREQ_TO_PSDI 0x00000002     /* RW--V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMREQ_TO_PSL0 0x00000004     /* RW--V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMREQ_TO_PSL1 0x00000006     /* RW--V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMREQ_TO_PSL2 0x00000008     /* RW--V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMSTS        7:4             /* R-EVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMSTS_NOP    0x00000000      /* R---V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMSTS_PSDI   0x00000002      /* R---V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMSTS_PSL0   0x00000004      /* R---V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMSTS_PSL1   0x00000006      /* R---V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMSTS_PSL2   0x00000008      /* R-E-V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMSTS_PSERR  0x0000000a      /* R---V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMBUSY       8:8             /* R-EVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMBUSY_OFF   0x00000000      /* R-E-V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PMBUSY_ON    0x00000001      /* R---V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PSL1TXSLEEPVAL 10:9          /* RWEVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PSL1TXSLEEPVAL_INIT 0x00000002 /* RWE-V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PSL1RXSLEEPVAL 12:11         /* RWEVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PSL1RXSLEEPVAL_INIT 0x00000003 /* RWE-V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PSDITXSLEEPVAL 14:13         /* RWEVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PSDITXSLEEPVAL_INIT 0x00000000 /* RWE-V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PSDIRXSLEEPVAL 16:15         /* RWEVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PSDIRXSLEEPVAL_INIT 0x00000000 /* RWE-V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_TXCLK_SWITCH_TIMEOUT_ERR 17:17 /* RWEVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_TXCLK_SWITCH_TIMEOUT_ERR__ONWRITE "oneToClear" /*       */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_TXCLK_SWITCH_TIMEOUT_ERR_OFF 0x00000000 /* RWE-V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_TXCLK_SWITCH_TIMEOUT_ERR_ON 0x00000001 /* RW--V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PHYARB_TIMEOUT_ERR 18:18     /* RWEVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PHYARB_TIMEOUT_ERR__ONWRITE "oneToClear" /*       */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PHYARB_TIMEOUT_ERR_OFF 0x00000000 /* RWE-V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_PHYARB_TIMEOUT_ERR_ON 0x00000001 /* RW--V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_INVLD_PMREQ_ERR 19:19        /* RWEVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_INVLD_PMREQ_ERR__ONWRITE "oneToClear" /*       */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_INVLD_PMREQ_ERR_OFF 0x00000000 /* RWE-V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_INVLD_PMREQ_ERR_ON 0x00000001 /* RW--V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_RSTPWRDN_DLY 20:20           /* RWEVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_RSTPWRDN_DLY_DIS 0x00000000  /* RWE-V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_RSTPWRDN_DLY_EN 0x00000001   /* RW--V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_RSTPWRDN_PSL2DLY 21:21       /* RWEVF */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_RSTPWRDN_PSL2DLY_DIS 0x00000000 /* RW--V */
#define NV_NVLPHYCTL_COMMON_PSAVE_UCODE_CTRL_STS_RSTPWRDN_PSL2DLY_EN 0x00000001 /* RWE-V */
#endif // __ls10_dev_nvlphyctl_ip_h__
