// This file is automatically generated by rmconfig - DO NOT EDIT!
//
// Provides RPC object boilerplate and RTTI.
//
// Profile:  shipping-gpus-openrm
// Haldef:   rpc.def
// Template: templates/gt_eng_odb.h
//

#ifndef _G_RPC_ODB_H_
#define _G_RPC_ODB_H_

#include "core/core.h"
#include "g_rpc_hal.h"


#define __RPC_OBJECT_BASE_DEFINITION \
    const struct NVOC_RTTI *__nvoc_rtti; \
    RPC_OBJ_IFACES obj; \
    RPC_HAL_IFACES _hal


extern const struct NVOC_CLASS_DEF __iom_class_def_OBJRPC;
#define __nvoc_class_def_OBJRPC __iom_class_def_OBJRPC

#ifndef __nvoc_class_id_OBJRPC
#define __nvoc_class_id_OBJRPC 0x1ab16a
#endif /* __nvoc_class_id_OBJRPC */


#endif  // _G_RPC_ODB_H_
